files = ["dist/Gaphor.app"]
symlinks = {"Applications": "/Applications"}
hide_extensions = ["Gaphor.app"]

window_rect = ((200, 120), (700, 400))
background = "_packaging/macos/background.png"

icon_size = 100
icon_locations = {
    "Gaphor.app": (200, 240),
    "Applications": (500, 240),
}
