/* Furo overrides */
body[data-theme="dark"] article img {
  background-color: var(--color-content-foreground);
}

@media (prefers-color-scheme: dark) {
  body:not([data-theme="light"]) article img {
    background-color: var(--color-content-foreground);
  }
}

/* Jupyter output */
.cell_output img {
  display: block;
}

.utterances {
  margin-top: 1rem;
}
